/* -*- Mode: C;  tab-width: 4;  indent-tabs-mode: dunto;  c-basic-offset: 4 -*-
*
* Copyright (C) 2019 Tianjin KYLIN Information Technology Co., Ltd.
*
* This program is free software;  you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation;  either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY;  without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.   See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program;  if not, write to the Free Software
* Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
*
*/

#include "activeconnectionwidget.h"
#include "config/xatom-helper.h"
#include "config/config.h"
#include <QPainterPath>

ActiveConnectionWidget::ActiveConnectionWidget(QString address, QString name, QString type, int rssi) :
    devAddr(address),
    devName(name),
    devType(type),
    devRSSI(rssi)
{
    if(QGSettings::isSchemaInstalled("org.ukui.style")){
        settings = new QGSettings("org.ukui.style");
        connect(settings,&QGSettings::changed,this,&ActiveConnectionWidget::GSettingsChanges);
    }

    const QByteArray transparency_id(TRANSPARENCY_SETTINGS);
    if(QGSettings::isSchemaInstalled(transparency_id)) {
        transparency = new QGSettings(transparency_id);
    }
    if(QGSettings::isSchemaInstalled(transparency_id)) {
        getTransparency();
        connect(transparency,&QGSettings::changed,this,&ActiveConnectionWidget::GSettingsChanges);
    }
    setProperty("useStyleWindowManager", false);
    setProperty("useSystemStyleBlur", true);
    this->setAttribute(Qt::WA_TranslucentBackground);
    //窗管协议
    MotifWmHints hints;
    hints.flags = MWM_HINTS_FUNCTIONS|MWM_HINTS_DECORATIONS;
    hints.functions = MWM_FUNC_ALL;
    hints.decorations = MWM_DECOR_BORDER;
    XAtomHelper::getInstance()->setWindowMotifHint(this->winId(), hints);

    this->setWindowIcon(QIcon::fromTheme("bluetooth"));
    this->setWindowTitle(tr("Bluetooth Connection"));

    this->setAttribute(Qt::WA_DeleteOnClose);
    this->setFixedSize(420, 154);

    window_icon = new QLabel(this);
    window_icon->setPixmap(QIcon::fromTheme("preferences-system-bluetooth").pixmap(22,22));

    window_title = new QLabel(tr("Bluetooth Connections"),this);
    window_title->setFixedSize(300,20);

    window_close = new QPushButton(this);
    window_close->setIcon(QIcon::fromTheme("window-close-symbolic"));
    window_close->setProperty("isWindowButton", 0x2);
    window_close->setProperty("useIconHighlightEffect", 0x8);
    window_close->setFlat(true);
    connect(window_close, &QPushButton::clicked, this, &ActiveConnectionWidget::onClick_close_btn);

    tip_txt = new QLabel(this);
    tip_txt->setFixedWidth(380);
    QFontMetrics fontMetrics(tip_txt->font());
    QString tip = QString(tr("Found audio device \"") + devName + tr("\", connect it or not?"));
    QString _tip = fontMetrics.elidedText(tip,
                                         Qt::ElideMiddle,
                                         tip_txt->width());
    if (tip != _tip)
        tip_txt->setToolTip(tip);
    tip_txt->setText(_tip);

    connect_btn = new QPushButton(tr("Connect"),this);
    connect_btn->setFocusPolicy(Qt::NoFocus);
    connect(connect_btn, &QPushButton::clicked, this, &ActiveConnectionWidget::onClick_connect_btn);
    cancel_btn = new QPushButton(tr("Cancel"),this);
    cancel_btn->setFocusPolicy(Qt::NoFocus);
    connect(cancel_btn, &QPushButton::clicked, this, &ActiveConnectionWidget::onClick_cancel_btn);

    window_icon->setGeometry(8,8,24,24);
    window_title->setGeometry(36, 10, 300, 20);
    window_close->setGeometry(388,8,24,24);
    tip_txt->setGeometry(24,48,380,28);
    cancel_btn->setGeometry(188,94,96,36);
    connect_btn->setGeometry(300,94,96,36);

    Config::setKdkGeometry(this->windowHandle(), this->width(),this->height(),false);
}

ActiveConnectionWidget::~ActiveConnectionWidget()
{
    qDebug() << Q_FUNC_INFO;
}

void ActiveConnectionWidget::getTransparency() {
    double proportion = transparency->get(TRANSPARENCY_KEY).toDouble();
    proportion = ((proportion > 0.85) ? 1 : (proportion + 0.15));
    qDebug() << Q_FUNC_INFO << proportion;
    tran = proportion * 255;
}

void ActiveConnectionWidget::onClick_close_btn()
{
    qDebug() << Q_FUNC_INFO << "close";
    onClick_cancel_btn();
}

void ActiveConnectionWidget::onClick_connect_btn()
{
    qDebug() << Q_FUNC_INFO << "accpet";
    emit replyActiveConnection(devAddr, true);
    this->hide();
    this->deleteLater();
}

void ActiveConnectionWidget::onClick_cancel_btn()
{
    qDebug() << Q_FUNC_INFO << "cancel";
    emit replyActiveConnection(devAddr, false);
    this->hide();
    this->deleteLater();
}

void ActiveConnectionWidget::GSettingsChanges(const QString &key)
{
    qDebug() << Q_FUNC_INFO << key;
    if(key == "styleName"){
        this->update();
    }
    if (key == "systemFontSize") {
        QFontMetrics fontMetrics(tip_txt->font());
        QString tip = fontMetrics.elidedText(QString(tr("Found audio device \"") + devName + tr("\", connect it or not?")),
                                             Qt::ElideMiddle,
                                             tip_txt->width());
        tip_txt->setText(tip);
    }
}

QPushButton* ActiveConnectionWidget::getButton() {
    switch (pressCnt) {
    case 1:
        return connect_btn;
    case 2:
        return cancel_btn;
    case 3:
        return window_close;
    default:
        return nullptr;
    }
}

void ActiveConnectionWidget::paintEvent(QPaintEvent *event) {
    QPainter pt(this);
    QColor color = QColor("#818181");
    pt.setPen(color);
    selectedBtn = getButton();
    if (selectedBtn == nullptr) {
        QWidget::paintEvent(event);
        return;
    }
    QRect rect = selectedBtn->geometry();
    pt.setRenderHint(QPainter::Antialiasing);
    pt.drawRect(rect);
    QWidget::paintEvent(event);
}

void ActiveConnectionWidget::keyPressEvent(QKeyEvent *event) {
    switch (event->key()) {
    case Qt::Key_Tab:
        qDebug() << "press Tab";
        ++pressCnt;
        if (pressCnt == 4)
            pressCnt = 1;
        this->update();
        break;
    case Qt::Key_Enter:
    case Qt::Key_Return:
        qDebug() << "press Enter";
        if (selectedBtn == nullptr)
            return;
        selectedBtn->clicked();
        break;
    case Qt::Key_Escape:
        qDebug() << "press Esc";
        if (pressCnt == 0)
            this->close();
        pressCnt = 0;
        this->update();
        break;
    default:
        break;
    }
}
